#!/usr/bin/env python

# This script calculates statistics of the percentage of translation per
# language for all languages in Ubuntu by calling all involved functions in the
# right order.
#
# Author: David Planella <david.planella@ubuntu.com>
#
# Copyright (c) 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import urllib
import os
from optparse import OptionParser
from common import utils
from ul10n_stats import ul10n_iso_pots_get
from ul10n_stats import ul10n_stats_calc
from ul10n_stats import ul10n_custom

DATA_DIR = 'data'
CONFIG_DIR = 'config'
STATS_URL = 'http://people.canonical.com/~danilo/ubuntu'

def options_parse():
    """Parses the command line and config file options"""

    settings = utils.get_settings()

    USAGE = "%prog [OPTIONS]"

    DESCRIPTION = """Lists all translation templates and source packages in the LiveCD for the
    given distro version."""

    parser = OptionParser(usage = USAGE, description = DESCRIPTION)

    parser.add_option("-d",
                    "--domains-file",
                    dest="domains_file",
                    default=None,
                    help="File to read domains from and on which stats are based. Requires -c argument.")

    parser.add_option("-c",
                    "--custom",
                    dest="custom",
                    action="store_true",
                    default=False,
                    help="Turn on custom (non-Ubuntu) mode")

    parser.add_option("-f",
                    "--no-file",
                    dest="no_file",
                    action="store_true",
                    default=False,
                    help="Disables using conf file blacklist.")

    parser.add_option("-p",
                    "--no-priorities",
                    dest="no_priorities",
                    action="store_true",
                    default=False,
                    help="Disables using priorities to generate blacklist.")

    parser.add_option("-s",
                    "--source-package-stats",
                    dest="pkg_stats",
                    default='{0}/{1}-package-stats.log.gz'.format(DATA_DIR, settings['distro_codename']),
                    help="Specify the file containing the source package statistics")

    parser.add_option("-b",
                    "--template-blacklist",
                    dest="blacklist",
                    default='config/blacklist.conf',
                    help="Specify the file containing the templates to blacklist during the calculation of statistics")

    (options, args) = parser.parse_args()

    return options, settings


def get_raw_data(distro_codename):

    print >>sys.stderr, "Fetching stats data..."
    for stats_file in ('potemplate', 'package'):
        url = '{0}/{1}-{2}-stats.log.gz'.format(STATS_URL, distro_codename, stats_file)
        path = os.path.join(DATA_DIR, os.path.basename(url))
        urllib.urlretrieve(url, path)


def get_working_templates(options, settings):

    distro_codename = settings['distro_codename']
    distro_id = settings['distro_id']
    TEMPLATES_FILE = '{0}/{1}-potemplate-stats.log.gz'.format(DATA_DIR, distro_codename)

    print >>sys.stderr, "Fetching working templates set..."

    # if normal ubuntu mode, get src pkg list form seeds, else from custom
    if not options.custom:
        # Get the source packages from seeds
        distro_src_pkgs = ul10n_iso_pots_get.get_src_pkgs_from_seeds(distro_id,
                                                                    distro_codename)

        # create list ('valid') of lines from lp_stats_pots file that only includes
        # src pkgs that are: in the seed, enabled, or in extra (for live cd)
        valid = ul10n_iso_pots_get.apply_valid_pots_filter(TEMPLATES_FILE, distro_src_pkgs)

        #for line in valid:
        #    print >> sys.stderr, line.split('|')[0].strip()

        working_pots = ul10n_iso_pots_get.apply_blacklist_filter(options, settings, valid)

        return working_pots

    else:
        domain_src, domain_linenum, domain_template, lines = ul10n_iso_pots_get.get_dictionaries(TEMPLATES_FILE)
        if options.domains_file:
            if not os.path.exists(options.domains_file):
                print >> sys.stderr, "Error: domains_file not found. Stopping."
                sys.exit(1)
            f = open(options.domains_file, 'r')
            domains = f.read().split('\n')
            f.close()
        else:
            domains = ul10n_custom.get_domains()
        for domain in domains: print >> sys.stderr, domain
        valid = list()
        invalid = list()
        for domain in domains:
            if domain in domain_src.keys():
                line = lines[domain_linenum[domain]].rstrip()
                valid.append(line)

        return valid


def main():

    # Get the command line options and settings from the config file
    options, settings = options_parse()

    # Fetch the raw data export files to be used as the source for the
    # calculation
    get_raw_data(settings['distro_codename'])

    # Compile a list of working templates to be considered for the calculation.
    # Apply any blacklisting if necessary.
    working_pots = get_working_templates(options, settings)

    # Calculate the unified, per language translation statistics from the
    # working templates set and output them in a report
    ul10n_stats_calc.calculate_stats(options, settings, working_pots)

if __name__ == "__main__":
    main()
    sys.exit(0)
