#!/usr/bin/env python

# livecd-pkgs-get - Provides a list of packages included in the LiveCD of a
# given distro
#
# Author: David Planella <david.planella@ubuntu.com> and
#   Kyle Nitzsche <kyle.nitzsche@canonical.com>
#
# Copyright (c) 2009, 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import re
import urllib2
import csv
import os
import gzip

def get_src_pkgs_from_seeds(distro_id, distro_codename):

    SEEDS_URL = 'http://people.canonical.com/~ubuntu-archive/germinate-output/%(distro_id)s.%(distro_codename)s' % {'distro_id': distro_id, 'distro_codename': distro_codename}
    SEEDS = ('boot', 'required', 'minimal', 'standard', 'desktop-common', 'desktop')

    src_pkgs = set()

    for seed in SEEDS:
        seed_url = '{0}/{1}'.format(SEEDS_URL, seed)
        seed_opener = urllib2.build_opener()
        try:
            seed_file = seed_opener.open(seed_url)
            seed = seed_file.read()
        except urllib2.URLError, exc:
            print "Error opening %s: %s" % (seed_url, exc)
            sys.exit()
        else:
            seed_reader = csv.reader(seed.splitlines(), delimiter='|')
            for row in seed_reader:
                first_row = row[0].strip()
                if (first_row != 'Package') and not (first_row.startswith('-') or first_row == ''):
                    src_pkgs.add(row[1].strip())

    return src_pkgs

def apply_valid_pots_filter(lp_stats_pots, src_pkgs_superset):

    lines = gzip.open(lp_stats_pots, 'rb').readlines()

    EXTRA_SRC_PKGS = '(debian-installer|ubiquity|ubiquity-slideshow-ubuntu|gfxboot-theme-ubuntu)'

    # Add additional templates in the LiveCd which are not installed (or
    # not enabled in LP but still important)
    extra_templates = re.compile(EXTRA_SRC_PKGS)

    valid = list()
    started = 1
    for line in lines:
        if started > 0:
            started -= 1
            continue

        if '|' in line:
            src_pkg = line.split('|')
            src_pkg_name = src_pkg[0].strip()
            src_pkg_is_enabled = (src_pkg[4].strip() == 't')
            if ((src_pkg_name in src_pkgs_superset) and src_pkg_is_enabled) or (extra_templates.search(src_pkg_name)):
                valid.append(line.strip())
    return valid

def apply_blacklist_filter(options, settings, pot_set):

    blacklist = options.blacklist

    # create or get blacklist
    priority_blacklist = list(tuple())

    if not options.no_priorities:
        # Set up blacklist from priorities
        type_msg = "Generated from Priorities"
        cutoff_priority = settings['blacklist_priority']

        for line in pot_set:
            items = line.split('|')
            priority = items[6].strip()
            if priority < cutoff_priority:
                src = items[0].strip()
                template = items[1].strip()
                priority_blacklist.append((src, template))
        print >>sys.stderr, "BLACKLISTED through Priority: " + str(len(priority_blacklist))

    write_blacklist(priority_blacklist, "priority", settings)

    file_blacklist = list(tuple())

    if not options.no_file:
        # Set up blacklist from config/blacklist.conf
        type_msg = "Read from Conf File"

        for line in open(blacklist):
            blk_pkg, blk_pot = line.split('|')
            blk_pkg = blk_pkg.strip()
            blk_pot = blk_pot.strip()
            file_blacklist.append((blk_pkg, blk_pot))
        print >>sys.stderr, "BLACKLISTED through File: " + str(len(file_blacklist))

    write_blacklist(file_blacklist, "file", settings)

    valid = list()
    started = 1
    for line in pot_set:
        if started > 0:
            started -= 1
            continue

        if '|' in line:
            (pkg, domain, pot, total, enabled, langpack, priority, date) = line.split('|')
            pkg = pkg.strip()
            pot = pot.strip()
            if ((pkg, pot) not in file_blacklist) and ((pkg, pot) not in priority_blacklist):
                valid.append(line.strip())
    return valid

def get_dictionaries(f):

    lines = gzip.open(f, 'r').readlines()

    domain_src = dict()
    domain_template = dict()
    domain_linenum = dict()

    line_num = -1
    started = 1
    for line in lines:
        line_num += 1
        if started > 0:
            started -= 1
            continue

        if '|' in line:
            items = line.split('|')
            src_pkg = items[0].strip()
            domain = items[1].strip()
            template = items[2].strip()
            if items[4].strip() == 't': # true when src pkg enabled
                if domain not in domain_src.keys():
                    domain_src[domain] = src_pkg
                if (domain, template) not in domain_template.keys():
                    domain_template[domain] = template
                if (domain,line_num) not in domain_linenum.keys():
                    domain_linenum[domain] = line_num
    return domain_src, domain_linenum, domain_template, lines

def write_blacklist(bl, msg, settings):
    # write used blacklist to file in data dir
        black_list_f = open(os.path.join("data", settings['distro_codename'] + "-black_listed-" + msg), 'w')
        black_list_f.write("- " + msg + " -\n- Source Package | Template -\n-----------------------------\n")
        bl_l = list(bl)
        bl_l.sort()
        for item in bl_l:
            black_list_f.write(item[0] + ' | ' + item[1] + '\n')
        black_list_f.flush()
        black_list_f.close()
