#!/usr/bin/env python

# ul10n_stats_calc - Calculates translation statistics for a given set of
# source packages and domains, and produces a report
#
# Author: Danilo Segan <danilo@canonical.com>, with modifications by
#   David Planella <david.planella@ubuntu.com>
#   Kyle Nitzsche <kyle.nitzsche@canonical.com>
#
# Copyright (c) 2009, 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import codecs
import os
import urllib2
import datetime
import tempfile
import atexit
import shutil
import gzip
from common import utils
from common.templates import html_report_template
sys.path.append(os.path.join(os.getcwd(), 'lib/python2.6/site-packages'))
from gviz_api import gviz_api


def addToLanguage(stats, language, translated, total):
    if language in stats:
        stats[language]['translated'] += translated
        stats[language]['total'] += total
    else:
        stats[language] = {
            'translated': translated,
            'total': total,
            }


def do_lp_login():

    from launchpadlib.launchpad import Launchpad

    cachedir = tempfile.mkdtemp()
    atexit.register(shutil.rmtree, cachedir)

    print >> sys.stderr, "Logging in to Launchpad... "

    # TODO: error checking, especially if we're working offline
    launchpad = Launchpad.login_anonymously('Ubuntu Translation stats',
            'production', cachedir)

    return launchpad


def get_lang_names():

    # Set only to False to speed up debugging
    lp_api = False

    lang_names = {}

    print >>sys.stderr, 'Getting language list...'

    if lp_api:
        max_tries = 2
        tries = 0

        launchpad = do_lp_login()

        try:
            langs = launchpad.languages
        except urllib2.URLError, e:
            tries += 1
            if tries >= max_tries or (e.code != 502 and e.code != 504):
                print >>sys.stderr, "Failed (%d): %s" % (e.code, url)
                raise

            print >>sys.stderr, "Retrying (%d): %s" % (e.code, url)
            time.sleep(3)

        for lang in langs:
            if lang.visible:
                lang_names[lang.code] = lang.english_name

        langs_len = len(langs)

    else:
        started = 2
        for line in open(os.path.join('data', 'langs.log')):
            if started > 0:
                started -= 1
            elif '|' in line:
                code, englishname, visible = line.split('|')
                code = code.strip()
                englishname = englishname.strip()
                visible = visible.strip() == "t"
                if visible:
                    lang_names[code] = englishname
        langs_len = len(lang_names)

    print >>sys.stderr, '{0} languages obtained'.format(str(langs_len))

    return lang_names


def print_report(variables, settings, custom):
    basename = '{0}-{1}-translation-stats.html'.format(settings['distro_id'], settings['distro_release'])
    report_filename = os.path.join('reports', basename)

    # Log the date of creation for the report
    gen_date = str(datetime.datetime.utcnow())

    # Add necessary variables for the report
    variables['gen_date'] = gen_date
    variables['distro_release'] = settings['distro_release']

    f = codecs.open(report_filename, 'w', 'utf-8')
    f.write(html_report_template.ubuntu_template % variables)
    f.close()

    print >>sys.stderr, 'Report generated at {0}'.format(report_filename)


def calculate_stats(options, settings, pot_set):

    # To hold all statistics.
    stats = {}

    # set up some vars
    pkg_stats = options.pkg_stats
    translated_percent = int(settings['translated_percent'])

    TOTAL = 0
    working_templates = set()

    for line in pot_set:
        (pkg, domain, pot, total, enabled, langpack, priority,
            date) = line.split('|')
        pkg = pkg.strip()
        pot = pot.strip()
        total = int(total.strip())
        working_templates.add((pkg, pot))
        TOTAL += total
    #print >> sys.stderr, "SRC PKGs: " + str(len(open(working_pot_stats).readlines()))
    print >> sys.stderr, "SRC PKGs AFTER BLACKLIST: " + str(len(working_templates))
    print >> sys.stderr, "TOTAL MESSAGES: " + str(TOTAL)

    lang_names = get_lang_names()

    # Skip 2 lines from the package stats file
    started = 2

    # Read all stats from the package stats file
    for line in gzip.open(pkg_stats, 'rb').readlines():
        if started > 0:
            started -= 1
        elif '|' in line:
            (pkg, domain, pot, date_pot, date_translation, lang, translated,
                unreviewed, changed, total) = line.split('|')
            pkg = pkg.strip()
            pot = pot.strip()
            lang = lang.strip()
            translated = int(translated.strip())
            total = int(total.strip())

            if (pkg, pot) in working_templates and lang in lang_names:
                addToLanguage(stats, lang, translated, total)
                ## Prints the per-language detailed template statistics
                ## Caveat: if there are no translations for a template in a
                ## given language, the template will not be listed
                #if lang == 'ca':
                #    print "{0:<32}| {1:<32}| {2:<32}| {3:<32} | {4:<32}".format(pkg, pot, str(translated), str(total - translated), str(total))
    
    # create langs and sort from most translated to least 
    langs = stats.keys()
    #print >> sys.stderr, "LANGS: " + str (langs)
    langs.sort(cmp=lambda a, b: cmp(stats[b]['translated'], stats[a]['translated']))
    #print >> sys.stderr, "LANGS, sorted: " + str(langs)

    translated_langs = []
    langs_list = []

    for lang in langs:
        if lang in lang_names:
            if stats[lang]['translated'] == 0:
                percent = 0
            else:
                percent = int(round(100.00 * stats[lang]['translated'] / TOTAL))
            langs_list.append({
                "language": lang_names[lang],
                "translated": int(percent)
                })
        if percent >= translated_percent:
            translated_langs.append(lang_names[lang])

    translated_langs.sort()

    total_translated_langs = str(len(translated_langs))
    translated_langs_str = "({0}) ".format(total_translated_langs) + ", ".join(translated_langs)
    total_langs = str(len(langs_list))

    keys = ("language", "translated")
    description = {"language": ("string", "Language"),
        "translated": ("number", "Translated")}

    # Load the data it into gviz_api.DataTable
    data_table = gviz_api.DataTable(description)
    data_table.LoadData(langs_list)

    # FIXME: If we do rounding of the percentages before here, languages with
    # the same percentage after rounding will be simply ordered alphabetically.
    # When losing the figures after the decimal point, we lose the order of
    # languages in a given percentage range
    languages_json = data_table.ToJSon(columns_order=("language", "translated"),
                                        order_by=[("translated", "desc"), ("language", "asc")])

    print_report(locals(), settings, options.custom)
